@@ -3,8 +3,6 @@ package DNS::Unbound::AsyncQuery::AnyEventXSPromises;
 use strict;
 use warnings;
 
-# This leaks; don’t use it.
-
 use AnyEvent::XSPromises ();
 
 our @ISA;
@@ -19,7 +17,7 @@ use constant _DEFERRED_CR => \&AnyEvent::XSPromises::deferred;
 
 my ($new, $class);
 
-sub _dns_unbound_then {
+sub _then {
 
     # This hackery is here because AE::XSP doesn’t accept subclassing.
 
